
local _,L = ...
local rematch = Rematch
local settings

-- scrollable options fields:
-- [1] = option type: "header" "check" or "button"
-- [2] = settings["VariableName"] (ie "AutoShow" or "LockDrawer")
-- [3] = localized name of setting as it appears listed in options
-- [4] = localized "tooltip" of setting as it appears in window at bottom of options
-- [5] = type-dependant: for "check" a "VariableName" means this option is dependant on that variable
rematch.optionsList = {
	{ "header", nil, L["Targeting Options"] },
	{ "check", "AutoShow", L["Auto show on target"], L["When targeting something with saved pets, and the pets are not already loaded, show the Rematch window and close it when pets load."] },
	{ "check", "AutoLoad", L["Auto load on target"], L["When a new target has saved pets not already loaded, load the pets immediately. Note: if you target by right-clicking, it may be too late to load pets!"] },
	{ "check", "AutoMouseover", L["On mouseover also"], L["Instead of waiting to target, auto-load pets when you mouseover something that has a saved team not already loaded."], "AutoLoad" },
	{ "check", "AutoLoadAlways", L["Always auto load"], L["In addition to auto loading at new targets, persistently auto load every time you target (or mouseover if mouseover option enabled)."], "AutoLoad" },
	{ "header", nil, L["Window Options"] },
	{ "check", "LargeWindow", L["Larger window"], L["Make the Rematch window larger for easier viewing."] },
	{ "check", "LargePetCard", L["Larger pet cards"], L["Make the pet cards and ability tooltips larger."] },
	{ "check", "LockWindow", L["Lock window"], L["Lock the Rematch window so it can't be dismissed with the Escape key or moved unless Shift is held."] },
	{ "check", "LockDrawer", L["Lock drawer"], L["Lock the pull-out drawer so it can't be dismissed with the Escape key."] },
	{ "check", "ShowSidePanel", L["Use side panel"], L["When the window is collapsed, add a pullout panel to the side to manipulate leveling pets and Auto Load status."] },
	{ "check", "StayForBattle", L["Stay for battle"], L["When a pet battle begins, keep Rematch on screen instead of hiding it. Note: the window will still close on player combat."] },
	{ "check", "PrioritizeBrowser", L["Close pet journal"], L["If the browser wants to open when opening Rematch, automatically close the default pet journal. The journal and browser can't be open at the same time."] },
	{ "header", nil, L["Loading Options"] },
	{ "check", "EmptyMissing", L["Empty missing pet slots"], L["When a team with missing pets loads and a pet is missing, empty the slot the pet would go to instead of ignoring the slot."] },
	{ "check", "KeepSummoned", L["Keep companion"], L["After a team is loaded, summon back the companion that was at your side before the load."] },
	{ "check", "KeepLeveling", L["Keep leveling pet"], L["If one of your current pets is a leveling pet, loading a team will not swap out that pet. Instead, only the first two saved pets will load alongside it."] },
	{ "check", "ShowOnInjured", L["Show on injured"], L["When pets load, show the window if any pets are injured. The window will show if any pets are dead or missing regardless of this setting."] },
	{ "header", nil, L["Leveling Queue Options"] },
	{ "check", "AutoConfirmFill", L["Auto confirm fill & empty"], L["Don't ask for confirmation when filling or emptying the leveling queue."] },
	{ "check", "KeepCurrentOnSort", L["Keep current pet on new sort"], L["Do not change the current leveling pet to the top-most pet when choosing a sort order."] },
	{ "check", "KeepQueueSort", L["Keep sort when emptied"], L["When the queue is emptied, preserve the sort order and auto rotate status instead of resetting them."] },
	{ "check", "HidePetToast", L["Hide pet toast"], L["Don't display the popup 'toast' when a new pet is automatically loaded from the leveling queue."] },
	{ "header", nil, L["Pet Browser Options"] },
	{ "check", "WideSearchBar", L["Wide search bar"], L["Reorganize the top of the pet browser to give a wider search bar."] },
	{ "check", "UseTypeBar", L["Use type bar"], L["Show the tabbed bar near the top of the pet browser to filter pet types, pets that are strong or tough vs chosen types."] },
	{ "check", "OnlyBattlePets", L["Only battle pets"], L["Never list pets that can't battle in the pet browser, such as Guild Heralds. Note: most filters like rarity, level or stats will not include non-battle pets already."] },
	{ "header", nil, MISCELLANEOUS },
	{ "check", "JumpToTeam", L["Jump to key"], L["While the mouse is over the team list or pet browser, hitting a key will jump to the next team or pet that begins with that letter."] },
	{ "check", "AutoConfirm", L["Auto confirm save & delete"], L["Don't ask for confirmation when saving over an existing team or deleting one."] },
	{ "check", "BestOfSpecies", L["Auto upgrade"], L["When learning a pet, automatically upgrade any lesser versions of the pet in your saved teams."] },
	{ "check", "DisableShare", L["Disable sharing"], L["Disable the Send button and also block any incoming pets sent by others. Import and Export still work."] },
	{ "check", "HideTooltips", L["Hide tooltips"], L["Hide the more common tooltips within Rematch."] },
	{ "check", "HideWarnings", L["Even alerts"], L["Also hide tooltips that warn when you can't place a pet somewhere. This is not recommended if you're new to the addon."], "HideTooltips" },
	{ "check", "UseMinimapButton", L["Use minimap button"], L["Place a button on the minimap to toggle Rematch."] },
	{ "check", "HideJournalButton", L["Hide journal button"], L["Do not place a Rematch button along the bottom of the default Pet Journal."] },
	{ "button", "RepairTeams", L["Rebuild Teams"], L["Fix any problems caused by changing petIDs or other weirdness with teams."] },
	{ "button", "BindingsButton", KEY_BINDINGS, L["Go to the key binding window to create or change bindings for Rematch."] },
}

rematch.ReflectSetting = {} -- table of functions to run to reflect changes for defined settings
rematch.OptionsListButtons = {} -- table of functions to run when a "button" option is clicked

function rematch:InitializeOptions()
	settings = RematchSettings
	local options = rematch.drawer.options
	options.header.text:SetText(L["Options"])
	options.version:SetText("Rematch version "..GetAddOnMetadata("Rematch","Version"))

	local scrollFrame = rematch.drawer.options.list.scrollFrame
	scrollFrame.update = rematch.UpdateOptionsList
	scrollFrame.stepSize = 40
	HybridScrollFrame_CreateButtons(scrollFrame,"RematchOptionsListTemplate")
	local scrollBar = scrollFrame.scrollBar
	scrollBar.doNotHide = true
	scrollBar.trackBG:SetPoint("TOPLEFT",-2,15)
	scrollBar.trackBG:SetPoint("BOTTOMRIGHT",2,-15)
	scrollBar.trackBG:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	scrollBar.trackBG:SetGradientAlpha("HORIZONTAL",.125,.125,.125,1,.05,.05,.05,1)

	-- go through each checkbutton, set its text and run its reflect if it has one
	local list = rematch.optionsList
	for i=1,#list do
		if rematch.ReflectSetting[list[i][2]] then
			rematch.ReflectSetting[list[i][2]]()
		end
	end

	-- if PBT is loaded, add a button to options to import teams
	if IsAddOnLoaded("PetBattleTeams") then
		table.insert(rematch.optionsList,{ "button", "PBTImportButton", L["Import from Pet Battle Teams"], L["Copy the teams from the addon Pet Battle Teams to Rematch."] })
	end

end

function rematch:OptionsOnShow()
	rematch.saved.header:Hide()
	rematch.current.header:Hide()
	rematch:UpdateOptionsList()
end

function rematch:OptionsOnHide()
	rematch.saved.header:Show()
	rematch.current.header:Show()
end

function rematch:RematchOptionsButton()
	rematch.drawer.options:Show()
end

function rematch:RematchOptionsOkayButton()
	rematch.drawer.options:Hide()
end

function rematch.ReflectSetting.AutoLoad()
	rematch.ReflectSetting.AutoMouseover() -- update whether UPDATE_UNIT_MOUSEOVER should be registered
	rematch.sidePanel.autoLoad:SetChecked(settings.AutoLoad)
end

function rematch.ReflectSetting.AutoMouseover()
	if settings.AutoMouseover and settings.AutoLoad then
		rematch:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	else
		rematch:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

function rematch.ReflectSetting.LargeWindow()
	-- instead of a straight SetScale, we'll adjust position so it stays at the same
	-- relative position on the screen, growing out/in from the bottomleft corner
	local oldScale = rematch:GetEffectiveScale()
	settings.XPos = rematch:GetLeft()
	settings.YPos = rematch:GetBottom()
	rematch:SetScale(settings.LargeWindow and 1.25 or 1.0)
	local newScale = rematch:GetEffectiveScale()
	settings.XPos = (settings.XPos*oldScale)/newScale
	settings.YPos = (settings.YPos*oldScale)/newScale
	rematch:ClearAllPoints()
	rematch:SetPoint("BOTTOMLEFT",settings.XPos,settings.YPos)
	rematch.ReflectSetting.LargePetCard()
end

function rematch.ReflectSetting.LargePetCard()
	RematchFloatingPetCard:SetScale((settings.LargeWindow and 1.2 or 1)+(settings.LargePetCard and .1 or 0))
	RematchAbilityCard:SetScale((settings.LargeWindow and 1 or .9)+(settings.LargePetCard and .1 or 0))
end


function rematch.ReflectSetting.BestOfSpecies()
	rematch:SetupBestOfSpecies()
end

function rematch:ToggleAutoLoad()
	settings.AutoLoad = not settings.AutoLoad
	if rematch.drawer.options:IsVisible() then
		rematch:UpdateOptionsList()
	else
		print(L["\124cffffff00Rematch Auto Load is now"],settings.AutoLoad and L["\124cff00ff00Enabled"] or L["\124cffff0000Disabled"])
	end
end

function rematch:UpdateOptionsList()
	local list = rematch.optionsList
	local numData = #list
	local scrollFrame = rematch.drawer.options.list.scrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	for i=1, #buttons do
		local index = i + offset
		local button = buttons[i]
		button.index = nil
		button:Hide()
		if ( index <= numData) then
			button.index = index
			button.name:ClearAllPoints()
			if list[index][1]=="header" then
				button.name:SetText(list[index][3])
				button.name:SetTextColor(1,.82,0)
				button.name:SetPoint("CENTER")
				button.back:Show()
				button.check:Hide()
				button.button:Hide()
			elseif list[index][1]=="check" then
				button.name:SetText(list[index][3])
				button.back:Hide()
				button.check:Show()
				button.button:Hide()
				button.check:SetPoint("LEFT",list[index][5] and 16 or 6,0)
				button.name:SetPoint("LEFT",button.check,"RIGHT",2,0)
				if list[index][5] and not settings[list[index][5]] then
					button.check:SetEnabled(0) -- disable dependant option if parent unchecked
					button.name:SetTextColor(.5,.5,.5)
				else
					button.check:SetEnabled(1)
					button.name:SetTextColor(1,1,1)
				end
				button.check:SetChecked(settings[list[index][2]])
			elseif list[index][1]=="button" then
				button.name:SetText("")
				button.button:SetText(list[index][3])
				button.back:Hide()
				button.check:Hide()
				button.button:Show()
			end
			button:Show()
		end
	end
	HybridScrollFrame_Update(scrollFrame,20*numData,20)
end

function Rematch:OptionsCheckOnEnter()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local tooltip = list[index][4]
		if tooltip then
			rematch.drawer.options.tooltip.text:SetText(tooltip)
		end
	end
end

function Rematch:OptionsCheckOnLeave()
	rematch.drawer.options.tooltip.text:SetText("")
end

function rematch:OptionsCheckOnClick()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local setting = list[index][2]
		settings[setting] = self:GetChecked()
		rematch:UpdateOptionsList()
		if rematch.ReflectSetting[setting] then
			rematch:HideDropDown()
			rematch.ReflectSetting[setting]()
		end
	end
end

function rematch:OptionsListButtonOnClick()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] and rematch.OptionsListButtons[list[index][2]] then
		rematch.OptionsListButtons[list[index][2]]()
	end
end

-- jumps to Rematch section in default key binding interface
function rematch.OptionsListButtons.BindingsButton()
	local offset
	local numBindings = GetNumBindings()
	for i=1,numBindings do
		if GetBinding(i)=="HEADER_REMATCH" then
			offset=i
			break
		end
	end
	if offset then
		LoadAddOn("Blizzard_BindingUI")
		ShowUIPanel(KeyBindingFrame)
		local limit = numBindings-KEY_BINDINGS_DISPLAYED
		offset = offset>limit and limit or max(0,offset-1)
		FauxScrollFrame_OnVerticalScroll(KeyBindingFrameScrollFrame, offset*KEY_BINDING_HEIGHT, KEY_BINDING_HEIGHT, KeyBindingFrame_Update)
  end
end

function rematch.OptionsListButtons.PBTImportButton()
	rematch:MigratePBT()
end

--[[ Minimap Button ]]

function rematch:CreateMinimapButton()
	if RematchMinimapButton then
		return
	end
	local button = CreateFrame("Button","RematchMinimapButton",Minimap)
	button:SetSize(31,31)
	button:SetToplevel(true)
	button:SetFrameStrata("MEDIUM")
	button:SetFrameLevel(button:GetFrameLevel()+3)
	button:RegisterForClicks("AnyUp")
	button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	button.icon = button:CreateTexture(nil,"BACKGROUND")
	button.icon:SetTexture("Interface\\Icons\\PetJournalPortrait")
	button.icon:SetSize(20,20)
	button.icon:SetPoint("CENTER")
	local border = button:CreateTexture(nil,"OVERLAY")
	border:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	border:SetSize(53,53)
	border:SetPoint("TOPLEFT")
	button:SetScript("OnMouseDown",rematch.MinimapButtonOnMouseDown)
	button:SetScript("OnMouseUp",rematch.MinimapButtonOnMouseUp)
	button:SetScript("OnClick",rematch.Toggle)
	button:SetScript("OnEnter",rematch.ToggleTooltip)
	button:SetScript("OnLeave",rematch.ToggleTooltipHide)
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart",rematch.MinimapButtonOnDragStart)
	button:SetScript("OnDragStop",rematch.MinimapButtonOnDragStop)
end

function rematch:MinimapButtonOnMouseDown()
	self.icon:SetPoint("CENTER",1,-1)
	self.icon:SetVertexColor(.75,.75,.75)
end

function rematch:MinimapButtonOnMouseUp()
	self.icon:SetPoint("CENTER")
	self.icon:SetVertexColor(1,1,1)
end

function rematch:MinimapButtonOnDragStart()
	self.update = self.update or CreateFrame("Frame",nil,self)
	self.update:SetScript("OnUpdate",rematch.MinimapButtonDragUpdate)
end

function rematch:MinimapButtonOnDragStop()
	rematch.MinimapButtonOnMouseUp(self) -- release button
	self.update:SetScript("OnUpdate",nil)
end

function rematch:MinimapButtonDragUpdate(elapsed)
	local x,y = GetCursorPosition()
	local minX,minY = Minimap:GetLeft(), Minimap:GetBottom()
	local scale = Minimap:GetEffectiveScale()
	settings.minimapButtonPosition = math.deg(math.atan2(y/scale-minY-70,minX-x/scale+70))
	rematch:MinimapButtonPosition()
end

function rematch:MinimapButtonPosition()
	local angle = settings.minimapButtonPosition or -162
	RematchMinimapButton:SetPoint("TOPLEFT",Minimap,"TOPLEFT",52-(80*cos(angle)),(80*sin(angle))-52)
end

function rematch.ReflectSetting.UseMinimapButton()
	if settings.UseMinimapButton then
		rematch:CreateMinimapButton()
		rematch:MinimapButtonPosition()
	end
	if RematchMinimapButton then
		RematchMinimapButton:SetShown(settings.UseMinimapButton)
	end
end

--[[ Rematch Pet Journal Button ]]

function rematch:CreateJournalButton()
	if RematchJournalButton then return end

	-- find the last frame anchored in a chain to the Find Battle button, if any
	local lastAnchor
	local function findAnchor(anchoredTo,...)
		lastAnchor = anchoredTo
		for i=1,select("#",...) do
			local button = select(i,...)
			if select(2,button:GetPoint())==anchoredTo then
				findAnchor(button,PetJournal:GetChildren())
				return
			end
		end
	end
	findAnchor(PetJournalFindBattle,PetJournal:GetChildren())

	-- create the button
	local button = CreateFrame("Button","RematchJournalButton",PetJournal,"UIPanelButtonTemplate") -- MagicButtonTemplate
	button:SetSize(100,22)
	button:SetText("Rematch")
	button:SetScript("OnClick",rematch.Toggle)
	button:SetScript("OnEnter",rematch.ToggleTooltip)
	button:SetScript("OnLeave",rematch.ToggleTooltipHide)

	-- position it anchored to the left of the lastAnchor found above
	button:SetPoint("RIGHT",lastAnchor,"LEFT",-2,0)
	-- now nudge it so it's properly centered if an addon does some yoffset
	local yoff = floor(button:GetBottom())-floor(PetJournal:GetBottom())-4
	button:SetPoint("RIGHT",lastAnchor,"LEFT",-2,-yoff)
end

function rematch.ReflectSetting.HideJournalButton()
	if PetJournal then
		local button = RematchJournalButton
		local hide = settings.HideJournalButton
		if button then
			button:SetShown(not hide)
		elseif not hide and not button then
			rematch:CreateJournalButton()
		end
	end
end

function rematch.ReflectSetting.UseTypeBar()
	rematch:UpdateTypeBar()
end

function rematch.ReflectSetting.WideSearchBar()
	local wide = settings.WideSearchBar
	rematch.browser.toggle:SetShown(not wide)
	rematch.browser.filterButton:SetShown(not wide)
	rematch.browser.smallFilterButton:SetShown(wide)
	rematch.browser.searchBox:SetWidth(wide and 178 or 124)
end

function rematch.ReflectSetting.OnlyBattlePets()
	rematch:UpdateBrowser()
end

-- The various methods of catching changing petIDs isn't working; will revamp in 3.0.
-- For now: rebuild team button in options will go through and rebuild all teams from
-- their speciesID, choosing the highest level first, and highest rarity when
-- equal level.
function rematch.OptionsListButtons.RepairTeams()

	local species = rematch.info -- table indexed by species of petIDs in this species
	local roster = rematch.roster
	local saved = RematchSaved

	rematch:BrowserOnShow() -- in case browser not shown, run its initialization
	roster:Update()
	rematch.browser:Hide()
	settings.browserOpen = nil
	HideUIPanel(PetJournalParent)

	wipe(species)
	-- gather all pets
	for i=1,roster:GetNumPets() do
		local petID = roster:GetPetByIndex(i)
		if type(petID)=="string" then
			local speciesID,_,level = C_PetJournal.GetPetInfoByPetID(petID)
			if speciesID then
				local _,_,_,_,rarity = C_PetJournal.GetPetStats(petID)
				table.insert(species,{speciesID,petID,level,rarity})
			end
		end
	end
	-- sort them by descending level and descending rarity
	table.sort(species,function(e1,e2)
		if e1 and not e2 then
			return true
		elseif e2 and not e1 then
			return true
		else
			if e1[3]>e2[3] then -- level descending
				return true
			elseif e1[3]<e2[3] then
				return false
			else -- levels are equal
				return e1[4]>e2[4] -- rarity descending
			end
		end
	end)

	-- function used below to find the first unused petID of a species
	local function findPetID(speciesID,pet1,pet2)
		for i=1,#species do
			local candidate = species[i][2]
			if species[i][1]==speciesID and candidate~=pet1 and candidate~=pet2 then
				return candidate
			end
		end
	end

	local teamCount = 0 -- number of teams tally
	local reviewCount = 0 -- number of teams that should be reviewed
	for teamName,team in pairs(saved) do
		if teamName~="~temp~" then
			local review
			for i=1,3 do
				local petID = team[i][1]
				if petID~=0 and petID~=rematch.emptyPetID then
					local speciesID = team[i][5] or petID
					if not speciesID then
						review = true
					else
						local oldPetID = team[i][1]
						team[i][1] = speciesID
						local newPetID = findPetID(speciesID,i>1 and team[1][1],i>2 and team[2][1])
						if newPetID then
							team[i][1] = newPetID
							if newPetID~=oldPetID then
								review = true
							end
						else
							review = true
						end
					end
				end
			end
			teamCount = teamCount + 1
			if review then
				if reviewCount==0 then
					print(L["\124cffffd200At least one pet in each of these Rematch teams was reassigned or not found:"])
				end
				print(teamName)
				reviewCount = reviewCount + 1
			end
		end
	end
	print(format(L["\124cffffd200%d/%d teams rebuilt."],reviewCount,teamCount))
end
